package net.wanji.com.netty.handler;

import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.com.cache.CrossInfoCache;
import net.wanji.com.cache.netty.NettyMessageCache;
import net.wanji.com.common.enums.HexSign;
import net.wanji.com.netty.agreementcommon.AgreementCommon;
import net.wanji.com.netty.result.CommandResultService;
import net.wanji.com.pojo.netty.MessageResultPojo;
import net.wanji.common.framework.spring.ServiceBeanContext;
import net.wanji.databus.po.CrossInfoPO;
import org.apache.commons.lang3.StringUtils;

import javax.annotation.Resource;
import java.util.Objects;

@Slf4j
@NoArgsConstructor
public class NettyServerHandler extends ChannelInboundHandlerAdapter {

    @Resource
    private CrossInfoCache crossInfoCache;
    /**
     * 建立连接后
     *
     * @param ctx
     * @throws Exception
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("{} connection is active....", ctx.channel().remoteAddress());
        ctx.fireChannelActive();
    }

    /**
     * 读
     *
     * @param ctx
     * @param msg
     * @throws Exception
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        String key = ctx.channel().remoteAddress().toString();
        String data = msg.toString();
        log.debug("receive server data :{}", data);

        String hexSign = AgreementCommon.getHexSign(data);
        if (StringUtils.isEmpty(hexSign)) {
            log.error("data unique hex sign error");
            return;
        }
        String className = HexSign.getClassNameByHexSign(hexSign);
        CommandResultService commandResultService = (CommandResultService) ServiceBeanContext.getBean(className);
        CrossInfoPO crossInfo = crossInfoCache.getCrossInfo(key);
        String hexResult = commandResultService.getCommandResult(crossInfo);
        if (Objects.nonNull(hexResult)) {
            setResultData(key, hexResult);
        }
    }


    private void setResultData(String key, String hexResult) {
        MessageResultPojo resultPojo = NettyMessageCache.NETTY_MESSAGE_RESULT_MAP.get(key);
        if (resultPojo != null) {
            resultPojo.setHexMessageResult(hexResult);
            resultPojo.getCountDownLatch().countDown();
        }
    }

    /**
     * 关闭
     *
     * @param ctx
     * @throws Exception
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        String s = ctx.channel().remoteAddress().toString();
        log.error("{} connection break", s);
        ctx.fireChannelInactive();
    }
}
